@@ -375,6 +375,7 @@ class AdapterLoader(WeightsLoader):
         load_as=None,
         loading_info=None,
         leave_out=None,
+        set_active=False,
         **kwargs
     ):
         """
@@ -425,7 +426,7 @@ class AdapterLoader(WeightsLoader):
         adapter_name = load_as or config["name"]
         # If the adapter is not part of the model, add it
         if adapter_name not in self.model.config.adapters.adapters:
-            self.model.add_adapter(adapter_name, config=config["config"])
+            self.model.add_adapter(adapter_name, config=config["config"], set_active=set_active)
         else:
             logger.warning("Overwriting existing adapter '{}'.".format(adapter_name))
 
@@ -460,7 +461,7 @@ class AdapterFusionLoader(WeightsLoader):
             "adapter_fusion_layer.{}".format(old_name), "adapter_fusion_layer.{}".format(new_name)
         )
 
-    def save(self, save_directory: str, name: str):
+    def save(self, save_directory: str, name: str, meta_dict=None):
         """
         Saves a AdapterFusion module into the given directory.
 
@@ -469,20 +470,19 @@ class AdapterFusionLoader(WeightsLoader):
             name (str, optional): The AdapterFusion name.
         """
 
-        if hasattr(self.model.config, "adapter_fusion_models"):
-            if name not in self.model.config.adapter_fusion_models:
-                if self.error_on_missing:
-                    raise ValueError(f"Unknown AdapterFusion '{name}'.")
-                else:
-                    logger.debug(f"No AdapterFusion with name '{name}' available.")
-                    return
+        if name not in self.model.config.adapters.fusions:
+            if self.error_on_missing:
+                raise ValueError(f"Unknown AdapterFusion '{name}'.")
+            else:
+                logger.debug(f"No AdapterFusion with name '{name}' available.")
+                return
 
         if not exists(save_directory):
             mkdir(save_directory)
         else:
             assert isdir(save_directory), "Saving path should be a directory where the head can be saved."
 
-        adapter_fusion_config = self.model.config.adapter_fusion
+        adapter_fusion_config = self.model.config.adapters.get_fusion(name)
 
         # Save the adapter fusion configuration
         config_dict = build_full_config(
@@ -492,13 +492,13 @@ class AdapterFusionLoader(WeightsLoader):
             model_name=self.model.model_name,
             model_class=self.model.__class__.__name__,
         )
-        self.weights_helper.save_weights_config(save_directory, config_dict)
+        self.weights_helper.save_weights_config(save_directory, config_dict, meta_dict=meta_dict)
 
         # Save head weights
         filter_func = self.filter_func(name)
         self.weights_helper.save_weights(save_directory, filter_func)
 
-    def load(self, save_directory, load_as=None, loading_info=None):
+    def load(self, save_directory, load_as=None, loading_info=None, **kwargs):
         """
         Loads a AdapterFusion module from the given directory.
 
@@ -518,13 +518,13 @@ class AdapterFusionLoader(WeightsLoader):
                 return None, None
 
         config = self.weights_helper.load_weights_config(save_directory)
-        if not hasattr(self.model.config, "adapter_fusion_models"):
-            self.model.config.adapter_fusion_models = []
 
         adapter_fusion_name = load_as or config["name"]
-        if adapter_fusion_name in self.model.config.adapter_fusion_models:
+        if adapter_fusion_name in self.model.config.adapters.fusions:
             logger.warning("Overwriting existing adapter fusion module '{}'".format(adapter_fusion_name))
-        self.model.add_adapter_fusion(adapter_fusion_name, config["config"])
+        self.model.add_adapter_fusion(
+            adapter_fusion_name, config["config"], overwrite_ok=True, set_active=kwargs.pop("set_active", True)
+        )
 
         # Load AdapterFusion weights
         filter_func = self.filter_func(adapter_fusion_name)
@@ -553,6 +553,22 @@ class PredictionHeadLoader(WeightsLoader):
         self.convert_to_flex_head = convert_to_flex_head
 
     def filter_func(self, head_name):
+        # ToDo remove this workaround
+        if self.model.__class__.__name__ == "T5ForConditionalGeneration":
+            if head_name:
+                return (
+                    lambda x: not x.startswith("encoder")
+                    and not x.startswith("decoder")
+                    and not x.startswith("shared")
+                    and "heads.{}".format(head_name) in x
+                )
+            else:
+                return (
+                    lambda x: not x.startswith("encoder")
+                    and not x.startswith("decoder")
+                    and not x.startswith("shared")
+                )
+
         if head_name:
             return lambda x: not x.startswith(self.model.base_model_prefix) and "heads.{}".format(head_name) in x
         else:
@@ -687,7 +703,9 @@ class PredictionHeadLoader(WeightsLoader):
                     head_config["id2label"] = {int(id_): label for label, id_ in custom_label2id.items()}
                     head_config["label2id"] = {label: int(id_) for label, id_ in custom_label2id.items()}
 
-                self.model.add_prediction_head_from_config(head_name, head_config, overwrite_ok=True)
+                self.model.add_prediction_head_from_config(
+                    head_name, head_config, overwrite_ok=True, set_active=kwargs.pop("set_active", True)
+                )
             # model with static head
             else:
                 if self.convert_to_flex_head:
