@@ -1,9 +1,11 @@
 import logging
+import os
 import warnings
 from abc import ABC, abstractmethod
 from os.path import join
 from typing import List, Optional, Union
 
+import torch
 from torch import nn
 
 from .composition import AdapterCompositionBlock, Fuse, Stack, parse_composition
@@ -11,7 +13,7 @@ from .configuration import AdapterConfig, AdapterFusionConfig, ModelAdaptersConf
 from .hub_mixin import PushAdapterToHubMixin
 from .loading import AdapterFusionLoader, AdapterLoader, PredictionHeadLoader, WeightsLoader
 from .modeling import Adapter, GLOWCouplingBlock, NICECouplingBlock
-from .utils import inherit_doc
+from .utils import EMBEDDING_FILE, TOKENIZER_PATH, inherit_doc
 
 
 logger = logging.getLogger(__name__)
@@ -111,6 +113,8 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
     def __init__(self, config, *args, **kwargs):
         super().__init__(config, *args, **kwargs)
         self.model_name = None
+        self.loaded_embeddings = {}
+        self._active_embedding = "default"
 
         # In some cases, the config is not an instance of a directly supported config class such as BertConfig.
         # Thus, we check the adapters config here to make sure everything is correct.
@@ -130,10 +134,12 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         for fusion_name in self.config.adapters.fusions:
             self._add_fusion_layer(fusion_name)
 
+        self.loaded_embeddings["default"] = self.get_input_embeddings()
+
     # These methods have to be implemented by every deriving class:
 
     @abstractmethod
-    def train_adapter(self, adapter_setup: Union[list, AdapterCompositionBlock]):
+    def train_adapter(self, adapter_setup: Union[list, AdapterCompositionBlock], train_embeddings=False):
         """Sets the model into mode for training the given adapters."""
         pass
 
@@ -232,7 +238,7 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
 
     def add_adapter_fusion(
         self,
-        adapter_names: Union[Fuse, list],
+        adapter_names: Union[Fuse, list, str],
         config=None,
         overwrite_ok: bool = False,
         set_active: bool = False,
@@ -241,18 +247,24 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         Adds AdapterFusion to the model with alll the necessary configurations and weight initializations
 
         Args:
-            adapter_names: a list of adapter names which should be fused
-            adapter_fusion_config (str or dict): adapter fusion configuration, can be either:
+            adapter_names (Fuse or list or str): AdapterFusion layer to add. Can be either:
+
+                - a ``Fuse`` composition block
+                - a list of adapter names to fuse
+                - a comma-separated string of adapter names to fuse
+            config (str or dict): adapter fusion configuration, can be either:
 
                 - a string identifying a pre-defined adapter fusion configuration
                 - a dictionary representing the adapter fusion configuration
                 - the path to a file containing the adapter fusion configuration
-            override_kwargs: dictionary items for values which should be overwritten in the default AdapterFusion configuration
+            overwrite_ok (bool, optional): Overwrite an AdapterFusion layer with the same name if it exists. By default (False), an exception is thrown.
             set_active (bool, optional): Activate the added AdapterFusion. By default (False), the AdapterFusion is added but not activated.
         """
-        # TODO-V2 Allow nested items or directly pass Fuse block?
         if isinstance(adapter_names, Fuse):
             adapter_names = adapter_names.children
+        elif isinstance(adapter_names, str):
+            adapter_names = adapter_names.split(",")
+
         if isinstance(config, dict):
             config = AdapterFusionConfig.from_dict(config)  # ensure config is ok and up-to-date
         # In case adapter already exists and we allow overwriting, explicitly delete the existing one first
@@ -281,19 +293,21 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         if self.active_adapters == Stack(adapter_name):
             self.active_adapters = None
 
-    def delete_adapter_fusion(self, adapter_names: Union[Fuse, list]):
+    def delete_adapter_fusion(self, adapter_names: Union[Fuse, list, str]):
         """
         Deletes the AdapterFusion layer of the specified adapters.
 
         Args:
-            adapter_names (Union[Fuse, list]): List of adapters for which to delete the AdapterFusion layer.
+            adapter_names (Union[Fuse, list, str]): AdapterFusion layer to delete.
         """
         if isinstance(adapter_names, Fuse):
             adapter_fusion_name = ",".join(adapter_names.children)
         elif isinstance(adapter_names, list):
             adapter_fusion_name = ",".join(adapter_names)
-        else:
+        elif isinstance(adapter_names, str):
             adapter_fusion_name = adapter_names
+        else:
+            raise ValueError("Invalid AdapterFusion definition: {}".format(adapter_names))
 
         if adapter_fusion_name not in self.config.adapters.fusions:
             logger.info("No AdapterFusion '%s' found for deletion. Skipping.", adapter_fusion_name)
@@ -332,28 +346,36 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
     def save_adapter_fusion(
         self,
         save_directory: str,
-        adapter_names: list,
+        adapter_names: Union[Fuse, list, str],
         meta_dict: dict = None,
         custom_weights_loaders: Optional[List[WeightsLoader]] = None,
     ):
         """
-        Saves an adapter and its configuration file to a directory so that it can be shared or reloaded using
-        `load_adapter()`.
+        Saves an AdapterFusion layer and its configuration file to a directory so that it can be shared or reloaded
+        using `load_adapter_fusion()`.
 
         Args:
-            save_directory (str): Path to a directory where the adapter should be saved.
-            adapter_name (str): Name of the adapter to be saved.
+            save_directory (str): Path to a directory where the AdapterFusion should be saved.
+            adapter_names (Union[Fuse, list, str]): AdapterFusion to be saved.
 
         Raises:
-            ValueError: If the given adapter name is invalid.
+            ValueError: If the given AdapterFusion name is invalid.
         """
+        if isinstance(adapter_names, Fuse):
+            adapter_fusion_name = ",".join(adapter_names.children)
+        elif isinstance(adapter_names, list):
+            adapter_fusion_name = ",".join(adapter_names)
+        elif isinstance(adapter_names, str):
+            adapter_fusion_name = adapter_names
+        else:
+            raise ValueError("Invalid AdapterFusion definition: {}".format(adapter_names))
 
         loader = AdapterFusionLoader(self)
-        loader.save(save_directory, adapter_names, meta_dict)
+        loader.save(save_directory, adapter_fusion_name, meta_dict)
         # save additional custom weights
         if custom_weights_loaders:
             for weights_loader in custom_weights_loaders:
-                weights_loader.save(save_directory, adapter_names)
+                weights_loader.save(save_directory, adapter_fusion_name)
 
     def load_adapter(
         self,
@@ -430,24 +452,16 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         **kwargs
     ) -> str:
         """
-        Loads a pre-trained pytorch adapter module from the local file system or a remote location.
+        Loads a pre-trained AdapterFusion layer from the local file system.
 
         Args:
-            adapter_fusion_name_or_path (str): can be either:
-
-                - the identifier of a pre-trained task adapter fusion module to be loaded from Adapter Hub
-                - a path to a directory containing adapter weights saved using `model.saved_adapter()`
-                - a URL pointing to a zip folder containing a saved adapter module
-            config (dict or str, optional): The requested configuration of the adapter fusion.
-                If not specified, will be either: - the default adapter config for the requested adapter fusion if
-                specified - the global default adapter fusion config
-            model_name (str, optional): The string identifier of the pre-trained model.
-            load_as (str, optional): Load the adapter using this name. By default, the name with which the adapter was
-                    saved will be used.
+            adapter_fusion_name_or_path (str): a path to a directory containing AdapterFusion weights saved using `model.save_adapter_fusion()`.
+            load_as (str, optional): Load the AdapterFusion using this name.
+                    By default, the name with which the AdapterFusion layer was saved will be used.
             set_active (bool, optional): Activate the loaded AdapterFusion. By default (False), the AdapterFusion is loaded but not activated.
 
         Returns:
-            str: The name with which the adapter was added to the model.
+            str: The name with which the AdapterFusion was added to the model.
         """
 
         loader = AdapterFusionLoader(self)
@@ -493,10 +507,11 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         custom_weights_loaders: Optional[List[WeightsLoader]] = None,
     ):
         """
-        Saves all adapters of this model together with their configuration to subfolders of the given location.
+        Saves all AdapterFusion layers of this model together with their configuration to subfolders of the given
+        location.
 
         Args:
-            save_directory (str): Path to a directory where the adapters should be saved.
+            save_directory (str): Path to a directory where the AdapterFusion layers should be saved.
         """
         for name in self.config.adapters.fusions:
             adapter_fusion_config = self.config.adapters.get_fusion(name)
@@ -528,6 +543,123 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
 
         self.config.adapters.is_parallelized = False
 
+    def load_embeddings(self, path: str, name: str):
+        """
+        Load a saved embedding from the given path. If the embedding was saved with a tokenizer it is returned
+
+        Args:
+            path: the path to the saved embedding
+            name: the name the embedding should be loaded as
+
+        Returns: a tokenizer if it ws saved with the embedding otherwise None
+
+        """
+        from ..models.auto.tokenization_auto import AutoTokenizer
+
+        if name in self.loaded_embeddings:
+            raise ValueError("An embedding with the name {} already exists".format(name))
+        tokenizer = None
+        tokenizer_path = os.path.join(path, TOKENIZER_PATH)
+        if os.path.isdir(tokenizer_path):
+            tokenizer = AutoTokenizer.from_pretrained(tokenizer_path)
+
+        embedding_path = os.path.join(path, EMBEDDING_FILE)
+        if not os.path.isfile(embedding_path):
+            raise FileNotFoundError("No embeddings found at {}".format(embedding_path))
+        weights = torch.load(embedding_path)
+
+        self.loaded_embeddings[name] = nn.Embedding.from_pretrained(weights)
+        self.set_active_embeddings(name)
+        return tokenizer
+
+    def add_embeddings(self, name, tokenizer, reference_embedding=None, reference_tokenizer=None, embedding_dim=None):
+        """
+        Add a new embedding to the model. If a reference embedding and reference tokenizer are provided tokens in the
+        present in both tokenizers are initialized to the embedding in the reference_embedding.
+
+        Args:
+            name: the name of the embedding
+            tokenizer: the tokenizer determining the vocab of the embedding
+            reference_embedding: the reference embedding to use for initializing the embeddings of tokens present in the newly created embedding
+            reference_tokenizer: the tokenizer providing the vocab for the reference embedding
+            embedding_dim: the dimension of the embeddings (if None the hidden_size from the config is used)
+
+        """
+        if name in self.loaded_embeddings:
+            raise ValueError("An embedding with the name {} already exists".format(name))
+        if embedding_dim is None:
+            embedding_dim = self.config.hidden_size
+        embedding = nn.Embedding(tokenizer.vocab_size, embedding_dim)
+        embedding.requires_grad_(False)
+        if (reference_embedding is not None and reference_tokenizer is None) or (
+            reference_tokenizer is not None and reference_embedding is None
+        ):
+            raise KeyError(
+                "Reference embedding and reference tokenizer are required to use initialize embeddings from reference embedding"
+            )
+        if reference_embedding is not None and reference_tokenizer is not None:
+            tokens = set(tokenizer.get_vocab().keys()) & set(reference_tokenizer.get_vocab().keys())
+            reference_vocab = reference_tokenizer.get_vocab()
+            vocab = tokenizer.get_vocab()
+            for t in tokens:
+                idx_reference = reference_vocab[t]
+                idx = vocab[t]
+                embedding.weight[idx] = self.loaded_embeddings[reference_embedding].weight[idx_reference].clone()
+        embedding.train(False)
+        self.loaded_embeddings[name] = embedding
+        self.set_active_embeddings(name)
+
+    def delete_embeddings(self, name):
+        """
+        Deletes the embedding with the given name
+
+        Args:
+            name: The name of the embedding that should be deleted
+
+        """
+        if name not in self.loaded_embeddings:
+            raise ValueError("No embedding with name {}".format(name))
+        if self.active_embeddings == name:
+            logger.warning("The active embedding is deleted. Setting the default embedding as active.")
+            self.set_active_embeddings("default")
+        del self.loaded_embeddings[name]
+
+    def save_embeddings(self, path, name, tokenizer=None):
+        """
+        Saves the embedding with the given name. If a tokenizer is passed as well the tokenizer is saved together with
+        the embedding.
+
+        Args:
+            path: The path where the embedding should be saved
+            name: The name of the embedding that should be saved
+            tokenizer: optionally a tokenizer to save with the embedding (default is None)
+
+        """
+        if self.active_embeddings == name:
+            self.loaded_embeddings[name] = self.get_input_embeddings()
+        os.makedirs(path, exist_ok=True)
+        embedding_path = os.path.join(path, EMBEDDING_FILE)
+        torch.save(self.loaded_embeddings[name].weight, embedding_path)
+        if tokenizer:
+            tokenizer_path = os.path.join(path, TOKENIZER_PATH)
+            tokenizer.save_pretrained(tokenizer_path)
+
+    def set_active_embeddings(self, name):
+        """
+        Sets the active embedding for the forward pass of the model
+
+        Args:
+            name: The name of the embedding that should be used
+
+        """
+        self.loaded_embeddings[self.active_embeddings] = self.get_input_embeddings()
+        self.set_input_embeddings(self.loaded_embeddings[name])
+        self._active_embedding = name
+
+    @property
+    def active_embeddings(self):
+        return self._active_embedding
+
 
 @inherit_doc
 class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
@@ -561,15 +693,15 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
         else:
             self.base_model.add_adapter(adapter_name, config, overwrite_ok=overwrite_ok, set_active=set_active)
 
-    def train_adapter(self, adapter_setup: Union[list, AdapterCompositionBlock]):
+    def train_adapter(self, adapter_setup: Union[list, AdapterCompositionBlock], train_embeddings=False):
         """
         Sets the model into mode for training the given adapters. If self.base_model is self, must inherit from a class
         that implements this method, to preclude infinite recursion
         """
         if self.base_model is self:
-            super().train_adapter(adapter_setup)
+            super().train_adapter(adapter_setup, train_embeddings)
         else:
-            self.base_model.train_adapter(adapter_setup)
+            self.base_model.train_adapter(adapter_setup, train_embeddings)
 
     def train_adapter_fusion(self, adapter_setup: Union[list, AdapterCompositionBlock], unfreeze_adapters=False):
         """
@@ -688,6 +820,59 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
             custom_weights_loaders=custom_weights_loaders,
         )
 
+    def save_adapter_fusion(
+        self,
+        save_directory: str,
+        adapter_names: Union[Fuse, list, str],
+        meta_dict: dict = None,
+        custom_weights_loaders: Optional[List[WeightsLoader]] = None,
+        with_head: Union[bool, str] = False,
+    ):
+        """
+        Saves an AdapterFusion layer and its configuration file to a directory so that it can be shared or reloaded
+        using `load_adapter_fusion()`.
+
+        Args:
+            save_directory (str): Path to a directory where the AdapterFusion should be saved.
+            adapter_names (Union[Fuse, list, str]): AdapterFusion to be saved.
+            with_head (Union[bool, str]): If True, will save a head with the same name as the AdapterFusionLayer. If a string,
+                this will be used as the name of the head to be saved.
+
+        Raises:
+            ValueError: If the given AdapterFusion name is invalid.
+        """
+        super().save_adapter_fusion(save_directory, adapter_names, meta_dict, custom_weights_loaders)
+
+        if with_head:
+            # Make sure to cover the different options for adapter_names
+            if isinstance(with_head, str):
+                head_name = with_head
+            elif isinstance(adapter_names, Fuse):
+                head_name = adapter_names.name
+            elif isinstance(adapter_names, list):
+                head_name = ",".join(adapter_names)
+            else:
+                head_name = adapter_names
+            if head_name not in self.heads:
+                raise ValueError("No head with name {} found".format(head_name))
+            loader = PredictionHeadLoader(self)
+            loader.save(save_directory, head_name)
+
+    def load_adapter_fusion(
+        self,
+        adapter_fusion_name_or_path: str,
+        load_as: str = None,
+        custom_weights_loaders: Optional[List[WeightsLoader]] = None,
+        set_active: bool = False,
+        with_head: bool = True,
+        **kwargs
+    ) -> str:
+        if with_head:
+            if custom_weights_loaders is None:
+                custom_weights_loaders = []
+            custom_weights_loaders.append(PredictionHeadLoader(self, error_on_missing=False))
+        super().load_adapter_fusion(adapter_fusion_name_or_path, load_as, custom_weights_loaders, set_active)
+
     def save_all_heads(self, save_directory):
         for head_name in self.heads:
             save_path = join(save_directory, head_name)
@@ -708,3 +893,35 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
             return super().get_adapter(name)
         else:
             return self.base_model.get_adapter(name)
+
+    def load_embeddings(self, path: str, name: str):
+        if self.base_model is self:
+            return super().load_embeddings(path, name)
+        else:
+            return self.base_model.load_embeddings(path, name)
+
+    def save_embeddings(self, path, name, tokenizer=None):
+        if self.base_model is self:
+            return super().save_embeddings(path, name, tokenizer)
+        else:
+            return self.base_model.save_embeddings(path, name, tokenizer)
+
+    def add_embeddings(self, name, tokenizer, reference_embedding=None, reference_tokenizer=None, embedding_dim=None):
+        if self.base_model is None:
+            return super().add_embeddings(name, tokenizer, reference_embedding, reference_tokenizer, embedding_dim)
+        else:
+            return self.base_model.add_embeddings(
+                name, tokenizer, reference_embedding, reference_tokenizer, embedding_dim
+            )
+
+    def set_active_embeddings(self, name):
+        if self.base_model is None:
+            return super().set_active_embeddings(name)
+        else:
+            return self.base_model.set_active_embeddings(name)
+
+    def delete_embeddings(self, name):
+        if self.base_model is None:
+            return super().delete_embeddings(name)
+        else:
+            return self.base_model.delete_embeddings(name)
