@@ -4,7 +4,7 @@ import torch
 from torch import nn
 
 from ..composition import AdapterCompositionBlock, parse_composition
-from ..heads import ClassificationHead, MultiLabelClassificationHead
+from ..heads import CausalLMHead, ClassificationHead, MultiLabelClassificationHead, TaggingHead
 from ..model_mixin import InvertibleAdaptersMixin, ModelAdaptersMixin
 from .bert import (
     BertEncoderAdaptersMixin,
@@ -50,6 +50,7 @@ class GPT2DecoderBlockAdaptersMixin(BertEncoderAdaptersMixin):
         self.output_adapters = GPT2OutputAdaptersModule(self)
         self.attention_adapters._init_adapter_modules()
         self.output_adapters._init_adapter_modules()
+        self.register_forward_pre_hook(self._adapter_block_pre_hook)
 
     def add_fusion_layer(self, adapter_names):
         self.attention_adapters.add_fusion_layer(adapter_names)
@@ -71,6 +72,13 @@ class GPT2DecoderBlockAdaptersMixin(BertEncoderAdaptersMixin):
         self.attention_adapters.enable_adapters(adapter_names, unfreeze_adapters, unfreeze_attention)
         self.output_adapters.enable_adapters(adapter_names, unfreeze_adapters, unfreeze_attention)
 
+    # Makes sure the "parent" reference always points to the correct module.
+    # This is especially relevant when using torch data parallelism.
+    @staticmethod
+    def _adapter_block_pre_hook(module, input_tensors):
+        object.__setattr__(module.attention_adapters, "parent", module)
+        object.__setattr__(module.output_adapters, "parent", module)
+
 
 class GPT2ModelAdapterMixin(InvertibleAdaptersMixin, ModelAdaptersMixin):
     def __init__(self, *args, **kwargs):
@@ -96,7 +104,7 @@ class GPT2ModelAdapterMixin(InvertibleAdaptersMixin, ModelAdaptersMixin):
 
         self.add_invertible_adapter(adapter_name)
 
-    def train_adapter(self, adapter_setup: Union[list, AdapterCompositionBlock]):
+    def train_adapter(self, adapter_setup: Union[list, AdapterCompositionBlock], train_embeddings=False):
         self.train()
         self.freeze_model(True)
         adapter_setup = parse_composition(adapter_setup)
@@ -104,6 +112,8 @@ class GPT2ModelAdapterMixin(InvertibleAdaptersMixin, ModelAdaptersMixin):
         self.enable_invertible_adapters(adapter_setup.flatten())
         # use the adapters to be trained by default in every forward pass
         self.set_active_adapters(adapter_setup)
+        if train_embeddings:
+            self.get_input_embeddings().train()
 
     def train_adapter_fusion(self, adapter_setup: Union[list, AdapterCompositionBlock], unfreeze_adapters=False):
         self.train()
@@ -176,6 +186,8 @@ class GPT2ModelHeadsMixin(ModelWithFlexibleHeadsAdaptersMixin):
     head_types = {
         "classification": ClassificationHead,
         "multilabel_classification": MultiLabelClassificationHead,
+        "causal_lm": CausalLMHead,
+        "tagging": TaggingHead,
     }
 
     def add_classification_head(
@@ -205,3 +217,14 @@ class GPT2ModelHeadsMixin(ModelWithFlexibleHeadsAdaptersMixin):
         else:
             head = ClassificationHead(self, head_name, num_labels, layers, activation_function, id2label)
         self.add_prediction_head(head, overwrite_ok)
+
+    def add_causal_lm_head(self, head_name, overwrite_ok=False):
+        """
+        Adds a causal language modeling head on top of the model.
+
+        Args:
+            head_name (str): The name of the head.
+            overwrite_ok (bool, optional): Force overwrite if a head with the same name exists. Defaults to False.
+        """
+        head = CausalLMHead(self, head_name)
+        self.add_prediction_head(head, overwrite_ok=overwrite_ok)
